/**
 *
 */
package pt.digitalis.maven.ormgenerator.utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.JarURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * File copying utilities that copy file/streams to file/streams.
 *
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Nov 13, 2007
 *
 */
public class FileUtils {

    /** the size of the internal buffer to copy streams */
    private static final int BUFFER_SIZE = 1024;

    /**
     * Copies an input stream content to a new file
     *
     * @param in
     *            the input stream
     * @param outFile
     *            the output file
     * @throws IOException
     *             if any IO error occurred
     */
    static public void copyFile(InputStream in, File outFile) throws IOException {

        copyStream(in, new FileOutputStream(outFile));
    }

    /**
     * Copy a list of resources loaded from the classloader to a target directory.
     *
     * @param sourcePath
     *            The source path of the resources
     * @param sourceFiles
     *            A list of resources to copy
     * @param targetDir
     *            The target directory to copy the resources to
     * @throws Exception
     *             the copying failed
     */
    public void copyFiles(String sourcePath, String[] sourceFiles, File targetDir) throws Exception {
        String sourceResourceName = null;
        File targetFile = null;
        InputStream is = null;
        FileOutputStream fos = null;

        try {
            targetDir.mkdirs();

            for (int i = 0; i < sourceFiles.length; i++) {
                sourceResourceName = sourcePath + sourceFiles[i];
                is = this.getClass().getResourceAsStream(sourceResourceName);
                targetFile = new File(targetDir, sourceFiles[i]);
                fos = new FileOutputStream(targetFile);

                copyStream(is, fos);

                is = null;
                fos = null;
            }
        } catch (Exception e) {
            if (is != null)
                is.close();
            if (fos != null)
                fos.close();
            throw e;
        }
    }

    /**
     * Copies the input stream to the output stream.
     *
     * @param is
     *            the source input stream
     * @param os
     *            the target output stream
     * @throws IOException
     *             the copying failed
     */
    static public void copyStream(InputStream is, OutputStream os) throws IOException {

        byte[] buf = new byte[BUFFER_SIZE];
        int n = 0;
        int total = 0;

        while ((n = is.read(buf)) > 0) {
            os.write(buf, 0, n);
            total += n;
        }

        is.close();
        os.flush();
        os.close();
    }

    /**
     * Returns the list of files inside a directory in a JAR file
     *
     * @param directory
     *            the directory URL
     * @return a list of JarFiles
     * @throws IOException
     *             if a resource could not be read
     */
    static public List<Resource> getFilesInDirectory(String directory) throws IOException {
        List<Resource> resources = new ArrayList<Resource>();
        Enumeration<URL> urls = Thread.currentThread().getContextClassLoader().getResources(directory);
        URL url;

        while (urls.hasMoreElements()) {
            url = urls.nextElement();

            if (url.getProtocol().equalsIgnoreCase("FILE")) {
                resources.add(new Resource(new File(URLDecoder.decode(url.getPath(), "UTF-8"))));

            } else if (url.getProtocol().equalsIgnoreCase("JAR")) {
                JarURLConnection conn = (JarURLConnection) url.openConnection();
                JarFile jar = conn.getJarFile();
                Enumeration<JarEntry> entries = jar.entries();

                while (entries.hasMoreElements()) {
                    JarEntry entry = entries.nextElement();

                    if (entry.getName().contains(directory))
                        resources.add(new Resource(jar, entry));
                }
            }
        }
        return resources;
    }

    /**
     * Transforms a dot notation path to a general path notation
     *
     * @param dotNotationString
     *            a String in the dot notation
     * @return A <code>String</code> in the path notation
     */
    public static String dotNotation2PathNotation(String dotNotationString) {
        String fileSeparator = null;
        if (File.separatorChar == '\\')
            fileSeparator = "\\\\";
        else
            fileSeparator = File.separator;
        String pathName = dotNotationString.replaceAll("\\.", fileSeparator);
        return pathName;
    }

    /**
     * Delete all files bellow the directory
     *
     * @param f
     *            File with the directory
     * @throws IOException
     */
    public static void deleteTree(File f) throws IOException {
        if (f.exists() && f.isDirectory()) {
            File[] fileList = f.listFiles();
            for (int i = 0; i < fileList.length; i++) {
                File fTemp = fileList[i];
                if (fTemp.isDirectory()) {
                    deleteTree(fTemp);
                }
            }
            f.delete();
        }
    }

    /**
     * Returns a file filter for a Java source files
     *
     * @return the filter
     */
    static public FilenameFilter getJavaSourcesFilter() {
        return new FilenameFilter() {
            public boolean accept(File b, String name) {
                return name.endsWith(".java");
            }
        };
    }

    /**
     * Returns a file filter for directories
     *
     * @return the filter
     */
    static public FilenameFilter getDirectoryFilter() {
        return new FilenameFilter() {
            public boolean accept(File b, String name) {
                return b.isDirectory();
            }
        };
    }
}
