/**
 * 
 */
package pt.digitalis.maven.ormgenerator.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

/**
 * Resource class. Represents a resource either being a normal file on disk or a JAR entry
 * 
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Nov 14, 2007
 * 
 */
public class Resource {

    /** Resource type */
    enum ResourceType {
        /** JarFile type */
        ZIPFILE_ENTRY,

        /** File type */
        FILE_ENTRY
    }

    /** the resource type */
    ResourceType type;

    /** The Jar file entry if the resource is a file inside a JAR file */
    ZipEntry zipEntry;

    /** The Jar file if the resource is a file inside a JAR file */
    JarFile jarFile;

    /** The file if the resource is a file on disk */
    File file;

    /**
     * Constructor for a JarFile
     * 
     * @param jarFile
     *            the JarFile object
     * 
     * @param zipEntry
     *            the resource as a JarFile object
     */
    public Resource(JarFile jarFile, ZipEntry zipEntry) {
        this.type = ResourceType.ZIPFILE_ENTRY;
        this.jarFile = jarFile;
        this.zipEntry = zipEntry;
        this.file = null;
    }

    /**
     * Constructor for a File
     * 
     * @param file
     *            the resource as a File object
     */
    public Resource(File file) {
        this.type = ResourceType.FILE_ENTRY;
        this.file = file;
        this.jarFile = null;
        this.zipEntry = null;
    }

    /**
     * Get an input stream for the resource
     * 
     * @return the inputStream to the resource
     * @throws IOException
     *             if the resource is not accessible
     */
    public InputStream getInputStream() throws IOException {
        if (type == ResourceType.FILE_ENTRY)
            return new FileInputStream(file);
        else
            return jarFile.getInputStream(zipEntry);
    }

    /**
     * Inspector that checks if the resource is a directory
     * 
     * @return T is the resource is a directory
     */
    public boolean isDirectory() {
        if (type == ResourceType.FILE_ENTRY)
            return file.isDirectory();
        else
            return zipEntry.isDirectory();
    }

    /**
     * Get the name of the contained resource
     * 
     * @return the name of the resource
     */
    public String getName() {
        if (type == ResourceType.FILE_ENTRY)
            return file.getAbsolutePath();
        else
            return zipEntry.getName();
    }
}
