package pt.digitalis.maven.ormgenerator;

import java.io.File;
import java.io.IOException;

import pt.digitalis.maven.ormgenerator.utils.FileUtils;
import pt.digitalis.maven.ormgenerator.utils.FreemarkerExecutor;
import freemarker.template.Configuration;

/**
 * @author Ant�nio Silva <a href="mailto:asilva@digitalis.pt">asilva@digitalis.pt</a>
 * @created 2007/11/21
 */
public abstract class BaseExecutor {


    /**
     * The extension of the properties files
     */
    public static final String PROPERTIES_EXTENSION = ".properties";

    /**
     * The auto package definition
     */
    public static final String AUTO_PACKAGE = ".auto";

    /**
     * The dao package definition
     */
    public static final String DAO_PACKAGE = ".dao";

    /**
     * The generated files prefix
     */
    public static final String AUTO_PREFFIX = "Auto";

    /**
     * The java file extension
     */
    public static final String JAVA_EXTENSION = ".java";

    /**
     * The default suffix of the DAO files
     */
    public static final String DAO_SUFFIX = "DAO";


    /**
     * The prefix given to the interfaces
     */
    public static final String INTERFACE_PREFFIX = "I";

    /**
     * The suffix given to the generated files
     */
    public static final String IMPL_FILE_SUFFIX = "Impl";

    /**
     * The suffix that will be added to the base package
     */
    public static final String IMPL_PACKAGE_SUFFIX = ".impl";


    /**
     * Freemarker configuration.
     */
    private Configuration freemarkerConfig = null;

    /**
     * The destination directory of the generated files
     */
    private String destinationDir = null;

    /**
     * The directory where the freemarker template files are located
     */
    private String templateDir = null;

    /**
     * The destination package of the generated files
     */
    private String packageName = null;

    /**
     * private method made private to prevent anyone instantiate this class without the initializations.
     */
    @SuppressWarnings("unused")
    private BaseExecutor() {
    }

    /**
     * Class constructor instanciates the <code>ServiceExecutor</code> with the required initializations
     *
     * @param packageName
     *            The base package for the generated files
     * @param templateDir
     *            Path for the template directory, could be relative or absolute.
     * @param destinationDir
     *            directory where the generated files should be inserted.
     */
    public BaseExecutor(String packageName, String templateDir, String destinationDir) {
        this.setDestinationDir(destinationDir);
        this.setPackageName(packageName);
        this.setTemplateDir(templateDir);
    }

    /**
     * Prepares the executor to run
     *
     * @param templateDir
     *            The directory where the templates are saved
     */
    public void prepareExecutor(String templateDir) {
        try {
            this.setFreemarkerConfig(FreemarkerExecutor.prepareFreemarker(this.getTemplateDir(), this.getClass()));
        } catch (IOException e) {
            this.setFreemarkerConfig(null);
        }
    }

    /**
     * Generates the files from the templates
     *
     * @return The result is <code>true</code> if the files were correctly generated well and false otherwise
     */
    public abstract boolean execute();

    /**
     * Obtains the path where the files will be saved
     *
     * @param baseDir
     *            The base directory name.
     * @param packageName
     *            The package name to be added to the base dir
     * @return A <code>String</code> with.
     */
    public static String obtainPath(String baseDir, String packageName) {
        String pathName = baseDir + File.separator + FileUtils.dotNotation2PathNotation(packageName);
        File f = new File(pathName);
        if (!f.exists())
            f.mkdirs();
        return pathName;
    }

    /**
     * @return the destinationDir
     */
    protected String getDestinationDir() {
        return destinationDir;
    }

    /**
     * @param destinationDir
     *            the destinationDir to set
     */
    private void setDestinationDir(String destinationDir) {
        this.destinationDir = destinationDir;
    }

    /**
     * @return the packageName
     */
    protected String getPackageName() {
        return packageName;
    }

    /**
     * @param packageName
     *            the packageName to set
     */
    private void setPackageName(String packageName) {
        this.packageName = packageName;
    }

    /**
     * @return the freemarkerConfig
     */
    protected Configuration getFreemarkerConfig() {
        return freemarkerConfig;
    }

    /**
     * @param freemarkerConfig
     *            the freemarkerConfig to set
     */
    private void setFreemarkerConfig(Configuration freemarkerConfig) {
        this.freemarkerConfig = freemarkerConfig;
    }

    /**
     * @return the templateDir
     */
    protected String getTemplateDir() {
        return templateDir;
    }

    /**
     * @param templateDir
     *            the templateDir to set
     */
    private void setTemplateDir(String templateDir) {
        this.templateDir = templateDir;
    }

}
