package pt.digitalis.maven.ormgenerator;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.maven.plugin.logging.Log;

import pt.digitalis.maven.ormgenerator.utils.FileUtils;
import pt.digitalis.maven.ormgenerator.utils.FreemarkerExecutor;
import freemarker.template.TemplateException;

/**
 * This class is responsible to generate the custom files for the ORM generation.<br/> The main process reads all the
 * already generated dao's and verifies if a custom DAO exists if it doesn't exists it creates one, so if a DAO has
 * already been customized it doesn't clean's it, maintaining the changes made by the user.
 *
 * @author Antnio Silva <a href="mailto:asilva@digitalis.pt">asilva@digitalis.pt</a>
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created 2007/11/20
 */
public class DAOUserExecutor extends BaseExecutor {

    /**
     * Name of the template used to create the interface
     */
    private static final String DAOINTERFACE_TEMPLATE = "daointerface.ftl";

    /**
     * Name of the template used to implementation
     */
    private static final String DAOHOME_TEMPLATE = "daohome.ftl";

    /**
     * The logger
     */
    private Log logger = null;

    /** If splitSchemas mode is active */
    private boolean splitSchemas;

    /**
     * Class constructor instantiates the <code>DAOUserExecutor</code> with the required initializations This
     * constructor only calls the super.
     *
     * @param packageName
     *            The base package for the generated files
     * @param templateDir
     *            Path for the template directory, could be relative or absolute.
     * @param destinationDir
     *            directory where the generated files should be inserted.
     * @param logger
     *            the log to use
     * @param splitSchemas
     *            T if split schemas active
     * @see pt.digitalis.maven.ormgenerator.BaseExecutor#BaseExecutor(String, String, String)
     */
    public DAOUserExecutor(String packageName, String templateDir, String destinationDir, Log logger,
            boolean splitSchemas)
    {
        super(packageName, templateDir, destinationDir);
        this.setLogger(logger);
        this.splitSchemas = splitSchemas;
    }

    /**
     * Generates the files from the templates
     *
     * @param generatedDest
     *            generated file destination dir
     * @return The result is <code>true</code> if the files were correctly generated well and false otherwise
     */
    public boolean generateFiles(String generatedDest)
    {
        boolean result = false;

        try
        {
            /* Obtains the necessary paths from the packages names */
            String pathAuto = obtainPath(generatedDest, this.getPackageName() + DAO_PACKAGE + AUTO_PACKAGE);
            String pathInterface = obtainPath(generatedDest, this.getPackageName() + DAO_PACKAGE);
            String pathImpl = obtainPath(generatedDest, this.getPackageName() + DAO_PACKAGE
                    + ServiceExecutor.IMPL_PACKAGE_SUFFIX);

            // obtains a list of generated dao
            List<File> pojoDirs = new ArrayList<File>();
            File mainDir = new File(pathAuto);
            pojoDirs.add(mainDir);

            if (splitSchemas)
            {
                // Search for first level sub-dirs (schema dirs)
                for (File schemaDir: mainDir.listFiles(FileUtils.getDirectoryFilter()))
                    pojoDirs.add(schemaDir);
            }

            for (File pojoDir: pojoDirs)
            {
                // For all dirs, search for java source files
                File[] pojoArr = pojoDir.listFiles(FileUtils.getJavaSourcesFilter());

                if (pojoArr != null)
                {
                    // iterates over the generated dao's and verifies if a custom dao has already been created
                    for (int i = 0; i < pojoArr.length; i++)
                    {

                        String pojoName = getBaseDAOName(pojoArr[i]);
                        String pathInterfaceWithSchema = pathInterface;
                        String schemaName = "";
                        String pathImplWithSchema = pathImpl;

                        if (splitSchemas && (pojoDir.getName() != mainDir.getName()))
                        {
                            pathInterfaceWithSchema = pathInterface + File.separator + pojoDir.getName();
                            pathImplWithSchema = pathImpl + File.separator + pojoDir.getName();
                            schemaName = "." + pojoDir.getName();
                        }

                        if (!userDAOExists(pojoName, pathInterfaceWithSchema, true))
                        {
                            File dir;

                            dir = new File(pathInterfaceWithSchema);
                            if (!dir.exists())
                                dir.mkdir();

                            // if a custom dao doesn't exist a new one will be created as it's implementation
                            Map<String, Object> map = new HashMap<String, Object>();
                            map.put("packageName", getPackageName());
                            map.put("schemaName", schemaName);
                            map.put("date", new Date());
                            map.put("declarationName", pojoName);

                            FreemarkerExecutor.processFreemarker(this.getFreemarkerConfig(), map,
                                    DAOINTERFACE_TEMPLATE, pathInterfaceWithSchema + File.separator + INTERFACE_PREFFIX
                                            + pojoName + DAO_SUFFIX + JAVA_EXTENSION);
                        }

                        if (!userDAOExists(pojoName, pathImplWithSchema, false))
                        {
                            File dir;

                            dir = new File(pathImplWithSchema);
                            if (!dir.exists())
                                dir.mkdir();

                            // if a custom dao doesn't exist a new one will be created as it's implementation
                            Map<String, Object> map = new HashMap<String, Object>();
                            map.put("packageName", getPackageName());
                            map.put("schemaName", schemaName);
                            map.put("date", new Date());
                            map.put("declarationName", pojoName);

                            FreemarkerExecutor.processFreemarker(this.getFreemarkerConfig(), map, DAOHOME_TEMPLATE,
                                    pathImplWithSchema + File.separator + pojoName + DAO_SUFFIX + "Impl"
                                            + JAVA_EXTENSION);
                        }
                    }
                }
            }
            result = true;

        }
        catch (IOException e)
        {
            this.getLogger().error(e.getMessage(), e);

        }
        catch (TemplateException e)
        {
            this.getLogger().error(e.getMessage(), e);

        }
        return result;
    }

    /**
     * Obtains the table name that of generated DAO from a file name
     *
     * @param file
     *            File that from will be extrated the tablename
     * @return Returns the table name from the dao
     */
    private String getBaseDAOName(File file)
    {
        String fullSuffix = DAO_SUFFIX + JAVA_EXTENSION;
        String fullPreffix = INTERFACE_PREFFIX + AUTO_PREFFIX;
        String pojoName = file.getName().substring(fullPreffix.length(), file.getName().length() - fullSuffix.length());
        return pojoName;
    }

    /**
     * Checks if a DAO with the name specified by <code>daoName</code> exists in the path given by
     * <code>pathInterface</code>
     *
     * @param daoName
     *            Name of the dao that will be searched.
     * @param path
     *            Path to the directory that where the dao should be
     * @param isInterface
     *            if is searching for an interface or implementation
     * @return true if the dao exists and false otherwise
     */
    private boolean userDAOExists(final String daoName, String path, boolean isInterface)
    {
        File fInteface;

        if (isInterface)
            fInteface = new File(path + File.separator + INTERFACE_PREFFIX + daoName + DAO_SUFFIX + JAVA_EXTENSION);
        else
            fInteface = new File(path + File.separator + daoName + DAO_SUFFIX + IMPL_FILE_SUFFIX + JAVA_EXTENSION);

        return fInteface.exists();
    }

    /**
     * The main execution process to generate the costum files
     *
     * @see pt.digitalis.maven.ormgenerator.BaseExecutor#execute()
     */
    @Override
    public boolean execute()
    {
        boolean result = false;
        this.prepareExecutor(this.getTemplateDir());
        this.generateFiles(this.getDestinationDir());
        result = true;
        return result;
    }

    /**
     * @return the logger
     */
    private Log getLogger()
    {
        return logger;
    }

    /**
     * @param logger
     *            the logger to set
     */
    private void setLogger(Log logger)
    {
        this.logger = logger;
    }

}
