/**
 *
 */
package pt.digitalis.maven.ormgenerator;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.maven.plugin.logging.Log;

import pt.digitalis.maven.ormgenerator.utils.FreemarkerExecutor;
import freemarker.template.TemplateException;

/**
 * Creates the factory class
 *
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Jun 20, 2008
 */
public class FactoryExecutor extends BaseExecutor {

    /** The name of the template for the factory */
    public static final String FACTORY_TEMPLATE = "factory.ftl";

    /** The logger */
    private Log logger = null;

    /** The data source name */
    private String dataSourceName;

    /**
     * Class constructor instantiates the <code>ServiceExecutor</code> with the required initializations
     *
     * @param packageName
     *            The base package for the generated files
     * @param dataSourceName
     *            The dataSource name
     * @param templateDir
     *            Path for the template directory, could be relative or absolute.
     * @param destinationDir
     *            directory where the generated files should be inserted.
     * @param baseCodeDIR
     *            the project base path
     * @param serviceConfigurationFileName
     *            Service configuration file name, this file should read from the resources
     * @param logger
     *            the Maven Plugin logger
     */
    public FactoryExecutor(String packageName, String dataSourceName, String templateDir, String destinationDir,
            String baseCodeDIR, String serviceConfigurationFileName, Log logger) {
        super(packageName, templateDir, destinationDir);
        this.setLogger(logger);
        this.dataSourceName = dataSourceName;
    }

    /**
     * This method is responsible to generate all the files.
     *
     * @return Returns <code>true</code> if the generation process went well and <code>false</code> otherwise
     */
    public boolean execute() {
        boolean result = true;

        // Configures the freemarker engine
        this.prepareExecutor(this.getDestinationDir());
        if (this.getFreemarkerConfig() == null) {
            this
                    .getLogger()
                    .error(
                            "The service executor process was unable to build the freemarker configuration. Please check if the template directory is in classpath  under the directory "
                                    + this.getTemplateDir());
            result = false;
        } else {
            // generates the files
            result = this.generateFactoryFiles(this.getDestinationDir());
        }

        return result;
    }

    /**
     * Generates the files from the templates
     *
     * @param generatedDest
     *            generated file destination dir
     * @return The result is <code>true</code> if the files were correctly generated well and false otherwise
     */
    public boolean generateFactoryFiles(String generatedDest) {
        // Get or create a template
        boolean result = false;

        try {

            String path = obtainPath(generatedDest, this.getPackageName()) + "/" + dataSourceName + "Factory.java";

            // Builds the data to be merged with the template
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("packageName", this.getPackageName());
            map.put("dataSourceName", dataSourceName);

            // Generates the factory class
            FreemarkerExecutor.processFreemarker(this.getFreemarkerConfig(), map, FACTORY_TEMPLATE, path);

            result = true;
        } catch (IOException e) {
            this.getLogger().error(e.getMessage(), e);
        } catch (TemplateException e) {
            this.getLogger().error(e.getMessage(), e);
        }
        return result;
    }

    /**
     * @return the logger
     */
    private Log getLogger() {
        return logger;
    }

    /**
     * @param logger
     *            the logger to set
     */
    private void setLogger(Log logger) {
        this.logger = logger;
    }
}
