package pt.digitalis.maven.ormgenerator;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactNotFoundException;
import org.apache.maven.artifact.resolver.ArtifactResolutionException;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.model.Repository;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

import pt.digitalis.maven.ormgenerator.hibernate.CustomReverseEngineeringStrategy;
import pt.digitalis.maven.ormgenerator.hibernate.CustomReverseEngineeringStrategySplitSchemas;
import pt.digitalis.maven.ormgenerator.utils.AntExecutor;
import pt.digitalis.maven.ormgenerator.utils.DAOFinder;
import pt.digitalis.maven.ormgenerator.utils.FileUtils;
import pt.digitalis.maven.ormgenerator.utils.Resource;

/**
 * GenerateModel task. Creates the model classes and mappings for the given configuration file. All configuration files
 * by convention if not specified are read from the config directory
 *
 * @goal run
 * @phase generate-sources requiresDependencyResolution runtime
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Aug 18, 2007
 */
public class GenerateModel extends AbstractMojo {

    /**
     * The plugin current version FIXME: Must find a way to get this at runtime!
     */
    static public final String PLUGIN_VERSION = "1.0.1-SNAPSHOT";

    /**
     * The Maven project object
     *
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    private MavenProject project;

    /**
     * The plugin dependencies.
     *
     * @parameter expression="${plugin.artifacts}"
     * @required
     * @readonly
     */
    private List<Artifact> pluginArtifacts;

    /**
     * The hibernate configuration file to use for the generation process
     */
    private File hibernateConfig;

    /**
     * The hibernate reverse engineering file to use for the generation process
     *
     * @parameter expression="${project.basedir}/src/main/config/hibernate.reveng.xml"
     */
    private File hibernateReveng;

    /**
     * The generation process configurator file. Defines the structure that will be created the inferred schemas.
     *
     * @parameter expression="${project.basedir}/src/main/config/orm.generation.xml"
     */
    private File ormGenerationConfig;

    /**
     * The dataSource name for the ORM
     *
     * @parameter
     */
    private String dataSourceName = "hibernate";

    /**
     * The base package where all files and package structure will be generated to
     *
     * @parameter
     */
    private String packageName;

    /**
     * Generates the classes ready for multi-instances usage
     *
     * @parameter default-value="false"
     */
    private boolean multiInstance;

    /**
     * Separates the generated classes in packages. One for each table schema.
     *
     * @parameter default-value="false"
     */
    private boolean splitSchemas;

    /**
     * @component
     */
    protected ArtifactFactory artifactFactory;

    /**
     * @component
     */
    protected ArtifactResolver resolver;

    /**
     * @parameter expression="${localRepository}"
     */
    protected ArtifactRepository localRepository;

    /**
     * @parameter expression="${project.remoteArtifactRepositories}"
     */
    protected List<Repository> remoteRepositories;

    /**
     * @see org.apache.maven.plugin.Mojo#execute()
     */
    public void execute() throws MojoExecutionException
    {

        hibernateConfig = new File(project.getBasedir().getAbsolutePath() + "/src/main/config/" + dataSourceName
                + ".cfg.xml");

        if (splitSchemas)
            this.getLog().info("Split Schemas Mode: Spliting geneated classes on packages for each database schema...");

        if (multiInstance)
            this.getLog().info("Multi Instance Mode: Generating sources for multi-instance access...");

        // Prepares the temporary script/template files for execution
        this.getLog().info("Configuring reveng templates...");
        prepareTempFiles();

        // Creates the ORM entities (POJOs and DAOs)
        this.getLog().info("Creating entities and auto DAOs...");
        createEntities();

        // Customize HBM files
        this.getLog().info("Customizing HBM files...");
        HBMCustomization.customizeHBMFiles(project.getBasedir().getAbsolutePath(), dataSourceName);

        // Creates the factory class
        this.getLog().info("Cleaning up unecessary generated classes...");
        cleanUpDAOsForIDs();

        // Creates custom files
        this.getLog().info("Creating missing custom user DAOs...");
        createCustomFiles();

        // Creates the access services and the corresponding IoC module if selected
        this.getLog().info("Creating services...");
        createServices();

        // Creates the factory class
        this.getLog().info("Creating factory and IoC bindings...");
        createFactory();
    }

    /**
     * Checks if a file with the given suffix exists on the given list
     *
     * @param allFiles
     *            the file list
     * @param fileName
     *            the suffix to match
     * @return True if it exists, False otherwise
     */
    private boolean hasNonIdClass(List<String> allFiles, String fileName)
    {

        String fileNameToSearch;

        if (fileName.endsWith("IdDAO.java"))
        {
            fileNameToSearch = fileName.replaceAll("IdDAO.java", "DAO.java");

            return allFiles.contains(fileNameToSearch);

        }
        else if (fileName.endsWith("IdDAOImpl.java"))
        {
            fileNameToSearch = fileName.replaceAll("IdDAOImpl.java", "DAOImpl.java");

            return allFiles.contains(fileNameToSearch);
        }

        return false;
    }

    /**
     * Deletes all DAOs that were erroneously generated for Data Id classes.
     */
    private void cleanUpDAOsForIDs()
    {
        String classesDir = BaseExecutor.obtainPath(project.getBasedir() + "/src/main/java", packageName);

        for (File pojoDir: DAOFinder.getSchemaDirsRecursive(classesDir))
        {
            File[] pojoArr = pojoDir.listFiles(FileUtils.getJavaSourcesFilter());

            if (pojoArr != null)
            {
                List<String> allFiles = new ArrayList<String>();

                for (File file: pojoArr)
                    allFiles.add(file.getName());

                for (int i = 0; i < pojoArr.length; i++)
                {
                    if ((pojoArr[i].getName().endsWith("IdDAO.java") && hasNonIdClass(allFiles, pojoArr[i].getName()))
                            || (pojoArr[i].getName().endsWith("IdDAOImpl.java") && hasNonIdClass(allFiles, pojoArr[i]
                                    .getName())))
                    {
                        pojoArr[i].delete();
                        getLog().debug("Deleted generated class: " + pojoArr[i].getName());
                    }
                }
            }
        }
    }

    /**
     * Prepares the temporary script/template files for execution
     *
     * @throws MojoExecutionException
     */
    private void prepareTempFiles() throws MojoExecutionException
    {
        // Copy the ant script and it's dependencies to the working directory (target) for execution
        File buildFile;

        // Copy the ant script files
        try
        {
            File buildDir = new File(project.getBasedir() + "/target");
            File log4jFile;

            // Create the target directory if it does not exist yet
            if (!buildDir.exists())
                buildDir.mkdir();

            buildFile = new File(project.getBasedir() + "/target/build.xml");
            log4jFile = new File(project.getBasedir() + "/target/log4j.properties");
            FileUtils.copyFile(this.getClass().getClassLoader().getResourceAsStream("build.xml"), buildFile);
            FileUtils.copyFile(this.getClass().getClassLoader().getResourceAsStream("log4j.properties"), log4jFile);

        }
        catch (IOException e)
        {
            throw new MojoExecutionException("Can't prepare internal ant script resource: " + e.getMessage(), e);
        }

        try
        {
            for (Resource resource: FileUtils.getFilesInDirectory("templates"))
            {

                File output = new File(project.getBasedir() + "/target/" + resource.getName());

                if (resource.isDirectory())
                    output.mkdir();
                else
                    FileUtils.copyFile(resource.getInputStream(), output);
            }

        }
        catch (IOException e)
        {
            throw new MojoExecutionException("Can't prepare internal template resource: " + e.getMessage(), e);
        }
    }

    /**
     * @return the current plugin Jar path
     */
    private String getPluginJarPath()
    {
        Artifact plugin = (Artifact) project.getPluginArtifactMap().get("pt.digitalis:maven-orm-generator-plugin");
        Artifact pluginArtifact = artifactFactory.createPluginArtifact(plugin.getGroupId(), plugin.getArtifactId(),
                plugin.getVersionRange());
        pluginArtifact.setVersion(GenerateModel.PLUGIN_VERSION);

        try
        {
            resolver.resolve(pluginArtifact, remoteRepositories, localRepository);
            if (pluginArtifact.getFile() != null)
            {
                try
                {
                    return pluginArtifact.getFile().toURL().toString().substring(5);

                }
                catch (MalformedURLException e)
                {
                    getLog().warn("Failed to convert " + pluginArtifact.getFile() + " to a URL", e);
                    return pluginArtifact.getFile().getAbsolutePath();

                }
            }
            else
                return pluginArtifact.getDownloadUrl();

        }
        catch (ArtifactResolutionException e)
        {
            getLog().info("Failed to resolve license artifact " + pluginArtifact, e);
            return "";

        }
        catch (ArtifactNotFoundException e)
        {
            if (getLog().isDebugEnabled())
                getLog().debug("License artifact " + pluginArtifact + " is not found", e);
            return null;
        }

    }

    /**
     * Creates the Entities for the model (POJO, hibernate mappings and DAOs)
     */
    private void createEntities()
    {

        File buildFile = new File(project.getBasedir() + "/target/build.xml");

        Properties properties = new Properties();
        properties.put("destination.package", packageName);
        properties.put("templates.dir", project.getBasedir() + "/target/templates");

        if (multiInstance)
            properties.put("templates.sufix", "/multi");
        else
            properties.put("templates.sufix", "/single");

        properties.put("config.file", hibernateConfig.getAbsolutePath());
        properties.put("reveng.file", hibernateReveng.getAbsolutePath());
        properties.put("dest.dir", project.getBasedir() + "/src/main/java");
        properties.put("hbm.dest.dir", project.getBasedir() + "/src/main/resources");
        properties.put("datasource.name", dataSourceName);
        properties.put("jar.path", getPluginJarPath());

        if (splitSchemas)
            properties.put("revenge.strategy.class", CustomReverseEngineeringStrategySplitSchemas.class
                    .getCanonicalName());
        else
            properties.put("revenge.strategy.class", CustomReverseEngineeringStrategy.class.getCanonicalName());

        AntExecutor.executeAntScript(buildFile, properties, pluginArtifacts, project, getLog());
    }

    /**
     * It generates the files that can be customized.
     *
     * @throws MojoExecutionException
     *             If the generation of the custom files wasn't complete.
     */
    private void createCustomFiles() throws MojoExecutionException
    {
        String templateDir = "/templates/userdao";

        if (multiInstance)
            templateDir += "/multi";
        else
            templateDir += "/single";

        String destinationDir = project.getBasedir() + "/src/main/java";
        DAOUserExecutor serExec = new DAOUserExecutor(packageName, templateDir, destinationDir, getLog(), splitSchemas);
        if (!serExec.execute())
            throw new MojoExecutionException("Unable to generate the Service files");
    }

    /**
     * Creates the services and IoC contributions
     *
     * @throws MojoExecutionException
     */
    private void createServices() throws MojoExecutionException
    {
        String templateDir = "/templates/service";

        if (multiInstance)
            templateDir += "/multi";
        else
            templateDir += "/single";

        String destinationDir = project.getBasedir() + "/src/main/java";
        String baseCodeDIR = project.getBasedir().getAbsolutePath();
        String configurationFile = ormGenerationConfig.getAbsolutePath();
        ServiceExecutor serExec = new ServiceExecutor(packageName, dataSourceName, templateDir, destinationDir,
                baseCodeDIR, configurationFile, multiInstance, getLog());
        if (!serExec.execute())
            throw new MojoExecutionException("Unable to generate the Service files");
    }

    /**
     * Creates the factory class
     *
     * @throws MojoExecutionException
     */
    private void createFactory() throws MojoExecutionException
    {
        String templateDir = "/templates/factory";

        if (multiInstance)
            templateDir += "/multi";
        else
            templateDir += "/single";

        String destinationDir = project.getBasedir() + "/src/main/java";
        String baseCodeDIR = project.getBasedir().getAbsolutePath();
        String configurationFile = ormGenerationConfig.getAbsolutePath();
        FactoryExecutor serExec = new FactoryExecutor(packageName, dataSourceName, templateDir, destinationDir,
                baseCodeDIR, configurationFile, getLog());
        if (!serExec.execute())
            throw new MojoExecutionException("Unable to generate the factory class");
    }
}
