package pt.digitalis.maven.ormgenerator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.maven.plugin.MojoExecutionException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * This class is responsible for manipulation the HBM files generate, to include settings that the reveng does not
 * allow.
 *
 * @author Luis Pinto <a href="mailto:lpinto@digitalis.pt">lpinto@digitalis.pt</a><br/>
 * @created Mar 17, 2010
 */
public class HBMCustomization {

    /** The HBM many-to-one attribute */
    public static final String MANY_TO_ONE_NODE = "many-to-one";

    /** The HMB class identifier */
    private static final String CLASS_ID = "<class ";
    /** The HMB many-to-one identifier */
    private static final String MANY_TO_ONE_ID = "<" + MANY_TO_ONE_NODE + " name=\"?\"";

    /** The HBM dynamic update class attribute */
    public static final String DYNAMIC_UPDATE_ATTRIBUTE = "dynamic-update";

    /**
     * Customize the HBM files
     *
     * @param projectBaseDir
     * @param dataSourceName
     *
     * @throws MojoExecutionException
     */
    public static void customizeHBMFiles(String projectBaseDir, String dataSourceName) throws MojoExecutionException
    {
        File hbmConfig = new File(projectBaseDir + "/src/main/config/" + dataSourceName + ".HBM.cfg.xml");

        if (hbmConfig.exists())
        {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            // Using factory get an instance of document builder
            DocumentBuilder db;
            try
            {
                db = dbf.newDocumentBuilder();
                // parse using builder to get DOM representation of the XML file
                Document dom = db.parse(hbmConfig.getAbsolutePath());

                // XML EXAMPLE
                // <class name="pt.digitalis.siges.model.data.css.PreReqCand">
                // <dynamic-update>true</dynamic-update>
                // <many-to-one name="periodoChamada">
                // <update>false</update>
                // <insert>false</insert>
                // </many-to-one>
                // </class>

                NodeList nl = dom.getElementsByTagName("class");

                if (nl != null && nl.getLength() > 0)
                {
                    for (int i = 0; i < nl.getLength(); i++)
                    {
                        Element classElement = (Element) nl.item(i);

                        String classFullName = classElement.getAttribute("name");

                        File hbmFile = new File(projectBaseDir + "/src/main/resources/"
                                + classFullName.replaceAll("\\.", "/") + ".hbm.xml");
                        if (hbmFile.exists())
                        {

                            Boolean dynamicUpdate = false;
                            List<String> manyToOneInsert = new ArrayList<String>();
                            List<String> manyToOneUpdate = new ArrayList<String>();

                            NodeList dynamicUpdateNodeList = classElement
                                    .getElementsByTagName(DYNAMIC_UPDATE_ATTRIBUTE);
                            dynamicUpdate = dynamicUpdateNodeList.getLength() > 0
                                    && "true".equalsIgnoreCase(dynamicUpdateNodeList.item(0).getTextContent());

                            NodeList manyToOneNodeNodes = classElement.getElementsByTagName(MANY_TO_ONE_NODE);
                            for (int j = 0; j < manyToOneNodeNodes.getLength(); j++)
                            {
                                Element manyToOneElement = (Element) manyToOneNodeNodes.item(j);
                                NodeList manyToOneChilds = manyToOneElement.getChildNodes();

                                for (int x = 0; x < manyToOneChilds.getLength(); x++)
                                {
                                    Node manyToOneChildNode = (Node) manyToOneChilds.item(x);

                                    if ("insert".equalsIgnoreCase(manyToOneChildNode.getNodeName())
                                            && "false".equalsIgnoreCase(manyToOneChildNode.getAttributes()
                                                    .getNamedItem("enable").getNodeValue()))
                                    {
                                        manyToOneInsert.add(manyToOneElement.getAttribute("name"));
                                    }
                                    if ("update".equalsIgnoreCase(manyToOneChildNode.getNodeName())
                                            && "false".equalsIgnoreCase(manyToOneChildNode.getAttributes()
                                                    .getNamedItem("enable").getNodeValue()))
                                    {
                                        manyToOneUpdate.add(manyToOneElement.getAttribute("name"));
                                    }
                                }
                            }

                            StringBuffer contents = new StringBuffer();
                            BufferedReader input = null;
                            try
                            {
                                input = new BufferedReader(new FileReader(hbmFile));
                                String line = null;

                                while ((line = input.readLine()) != null)
                                {
                                    StringBuffer buffer = new StringBuffer(line);
                                    if (buffer.indexOf(CLASS_ID) > -1 && dynamicUpdate)
                                    {
                                        buffer.insert(buffer.indexOf(CLASS_ID) + CLASS_ID.length(), " "
                                                + DYNAMIC_UPDATE_ATTRIBUTE + "=\"true\" ");
                                    }

                                    for (String manyToOneToInsert: manyToOneInsert)
                                    {
                                        String manyToOne = MANY_TO_ONE_ID.replace("?", manyToOneToInsert);
                                        if (buffer.indexOf(manyToOne) > -1)
                                        {
                                            buffer.insert(buffer.indexOf(manyToOne) + manyToOne.length(),
                                                    " insert=\"false\" ");
                                        }
                                    }

                                    for (String manyToOneToUpdate: manyToOneUpdate)
                                    {
                                        String manyToOne = MANY_TO_ONE_ID.replace("?", manyToOneToUpdate);
                                        if (buffer.indexOf(manyToOne) > -1)
                                        {
                                            buffer.insert(buffer.indexOf(manyToOne) + manyToOne.length(),
                                                    " update=\"false\" ");
                                        }
                                    }

                                    contents.append(buffer.toString() + System.getProperty("line.separator"));
                                }

                            }
                            catch (IOException ex)
                            {
                                contents = null;
                                ex.printStackTrace();

                            }
                            finally
                            {
                                try
                                {
                                    if (input != null)
                                        input.close();
                                }
                                catch (IOException ex)
                                {
                                    ex.printStackTrace();
                                }
                            }

                            // Not null. file exists and context parsed/updated
                            if (contents != null)
                            {
                                Writer output = null;

                                try
                                {
                                    output = new BufferedWriter(new FileWriter(hbmFile));
                                    output.write(contents.toString());

                                }
                                catch (IOException ex)
                                {
                                    ex.printStackTrace();
                                }
                                finally
                                {
                                    try
                                    {
                                        if (output != null)
                                            output.close();

                                    }
                                    catch (IOException ex)
                                    {
                                        ex.printStackTrace();
                                    }
                                }
                            }

                        }
                        else
                        {
                            System.err.println("The HBM file \"" + hbmFile.getAbsolutePath() + "\" doesn't exist");
                        }

                    }
                }
            }
            catch (ParserConfigurationException e)
            {
                throw new MojoExecutionException("Unable to personalize HBM Files");
            }
            catch (SAXException e)
            {
                throw new MojoExecutionException("Unable to personalize HBM Files");
            }
            catch (IOException e)
            {
                throw new MojoExecutionException("Unable to personalize HBM Files");
            }
        }
    }
}
